;; open X86

(* CDecl Calling Conventions:
 *    [ Caller's]              LARGER ADDRESSES
 *    [  Frame  ]  
 *    [Arg2     ]     EBP+12  = EBP + (2 + 1)*4
 *    [Arg1     ]     EBP+8   = EBP + (1 + 1)*4
 *    [Return   ]     EBP+4
 *    [ old EBP ]  <- EBP     (* Start of the Callee's frame *)
 *    [ locals  ]     EBP-4
 *
 *                             SMALLER ADDRESSES
 *)

let ebp_offset (amt:int32) : opnd =
  Ind{i_base = Some Ebp;
      i_iscl = None;
      i_disp = Some (DImm amt)} 


let square_block = 
  mk_block "square" [
    Mov(eax, stack_offset 4l);
    Imul(Eax, eax);
    Ret;
  ]

let f_block =
  mk_block "f" [
    Push(ebp);
    Mov(ebp, esp);
    Mov(eax, ebp_offset 12l); (* x2 *)
    Mov(ebx, ebp_offset 8l);  (* x1 *)
    Sub(eax, ebx);
    Push(eax);
    Call(Lbl (mk_lbl_named "square"));
    Add(esp, Imm 4l);  (* pop the argument *)
    Pop(ebp);    
    Ret;
  ]


let main_block = {
  global = true;  (* Make this block visible to the linker *)
  label = mk_lbl_named "_program";
  insns = [
    Push(ebp);
    Mov(ebp, esp);
    Push(Imm 12l);
    Push(Imm 3l);
    Call(Lbl (mk_lbl_named "f")); 
    Add(esp, Imm 8l);
    Pop(ebp);
    Ret;
  ];
}




let program =
  List.map (fun x -> Cunit.Code x) [main_block; f_block; square_block] 

let _ = Cunit.output_cunit program stdout

